@@ -1,3 +1,6 @@
+"""Apply perturbations to TextInstances and/or strings, generating one or many new instances."""
+
+
 import numpy as np
 import itertools
 import copy
@@ -316,7 +319,7 @@ class OneToOnePerturbation(Perturbation):
             >>> OneToOnePerturbation.from_string(replacement='UNKWRDZ')
 
         Raises:
-            AssertionError: At least one of `prefix`, `suffix` and `replacement` should be provided.
+            ValueError: At least one of `prefix`, `suffix` and `replacement` should be provided.
 
         Args:
             label_from (LT): Attribute label of original instance. Defaults to 'original'.
@@ -330,8 +333,8 @@ class OneToOnePerturbation(Perturbation):
             connector_after (Optional[str], optional): Overrides connector between `instance.data` and `suffix`, 
                 if it is None `connector` is used. Defaults to None.
         """
-        assert prefix is not None or suffix is not None or replacement is not None, \
-            'At least one of `prefix`, `suffix` and `replacement` should be provided.'
+        if prefix is None and suffix is None and replacement is None:
+            raise ValueError('At least one of prefix, suffix and replacement should be provided.')
 
         if prefix is None:
             connector_before = ''
@@ -411,7 +414,7 @@ class OneToOnePerturbation(Perturbation):
         """
         res = self.perturbation_function(instance)
 
-        if res is None:
+        if res is None or isinstance(res, list) and all(r is None for r in res):
             return
 
         perturbed, original_labels, labels = res
@@ -421,7 +424,7 @@ class OneToOnePerturbation(Perturbation):
         labels = as_list(labels)
 
         for perturbed_text, original_label, label in zip(perturbed, original_labels, labels):
-            if perturbed_text != str(instance.data):
+            if perturbed_text is not None and perturbed_text != str(instance.data):
                 identifier = format_identifier(instance, 0)
                 yield (MemoryTextInstance(identifier, perturbed_text, None),
                        [(instance.identifier, frozenset({original_label})),
@@ -446,7 +449,8 @@ class OneToManyPerturbation(Perturbation):
                       function: Callable[[str], Optional[Union[str, Sequence[str]]]],
                       label_from: LT = 'original',
                       label_to: LT = 'perturbed',
-                      n: int = 10):
+                      n: int = 10,
+                      perform_once: bool = False):
         """Construct a `OneToManyPerturbation` from a perturbation applied to a string.
 
         Args:
@@ -455,11 +459,14 @@ class OneToManyPerturbation(Perturbation):
             label_from (LT, optional): Attribute label of original instance. Defaults to 'original'.
             label_to (LT, optional): Attribute label of perturbed instance. Defaults to 'perturbed'.
             n (int, optional): Number of instances to generate. Defaults to 10.
+            perform_once (bool, optional): If the n parameter is in class construction perform once. Defaults to False.
         """
         import inspect
 
         if 'n' in inspect.signature(function).parameters:
             return super().from_function(lambda x: function(x, n=n), label_from=label_from, label_to=label_to)
+        elif perform_once:
+            return super().from_function(function, label_from=label_from, label_to=label_to)
 
         def perform_n_times(instance):
             perturbed = list(filter(None, [function(instance) for _ in range(n)]))
@@ -545,7 +552,7 @@ class OneToManyPerturbation(Perturbation):
             except:  # noqa: E722
                 return None
 
-        return cls.from_function(perturbation_function, n=n, label_from=label_from, label_to=label_to)
+        return cls.from_function(perturbation_function, perform_once=True, label_from=label_from, label_to=label_to)
 
     @text_instance
     def perturb(self, instance: TextInstance) -> Optional[Sequence[Tuple[TextInstance, Sequence[Tuple[KT, LT]]]]]:
@@ -563,7 +570,7 @@ class OneToManyPerturbation(Perturbation):
         """
         res = self.perturbation_function(instance)
 
-        if res is None:
+        if res is None or isinstance(res, list) and all(r is None for r in res):
             return
 
         perturbed, original_label, labels = res
